<?php

/**
 * @file
 * The PoolParty PPT API-class.
 */

class SemanticConnectorPPTApi {

  protected $connection;

  /**
   * The constructor of the PoolParty Thesaurus class.
   *
   * @param string $endpoint
   *   URL of the endpoint of the PoolParty-server.
   * @param string $credentials
   *   Username and password if required (format: "username:password").
   */
  public function __construct($endpoint, $credentials = '') {
    $this->connection = new SemanticConnectorCurlConnection($endpoint, $credentials);
  }

  /**
   * Get the configured cURL-connection.
   *
   * @return SemanticConnectorCurlConnection
   *   The connection object.
   */
  public function getConnection() {
    return $this->connection;
  }

  /**
   * Get a list of available projects of a PoolParty server.
   *
   * @return bool
   *   TRUE if the service is available, FALSE if not.
   */
  public function getProjects() {
    // Offer the possibility to support a different value for this function.
    $projects = NULL;

    $resource_path = '/PoolParty/api/projects';
    $result = $this->connection->get($resource_path);
    $projects = json_decode($result);

    if (!is_null($projects)) {
      foreach ($projects as &$project) {
        if (isset($project->uriSupplement)) {
          $project->sparql_endpoint_url = $this->connection->getEndpoint() . '/PoolParty/sparql/' . $project->uriSupplement;
        }
      }
    }
    else {
      $projects = array();
    }

    return $projects;
  }
}
